﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Facebook;
using FirstFacebookApplication;

namespace FacebookCSharpSDK_BatchRequests
{
    public partial class Form1 : Form
    {
        private const string AppId = "{appid}";
        private string[] extendedPermissions = new[] { "user_about_me", "offline_access", "publish_stream" };

        public Form1()
        {
            LoginToFacebook();
            InitializeComponent();
        }

        private void LoginToFacebook()
        {
            var fbLoginDialog = new FacebookLoginDialog(AppId, extendedPermissions, true);
            fbLoginDialog.ShowDialog();

            DisplayAppropriateMessage(fbLoginDialog.FacebookOAuthResult);
        }

        private void DisplayAppropriateMessage(FacebookOAuthResult facebookOAuthResult)
        {
            if (facebookOAuthResult != null)
            {
                if (facebookOAuthResult.IsSuccess)
                {
                    ExecuteBatchRequests(facebookOAuthResult.AccessToken);
                }
                else
                {
                    MessageBox.Show(facebookOAuthResult.ErrorDescription);
                }
            }
        }

        private void ExecuteBatchRequests(string accessToken)
        {
            ExecuteSyncBatchRequests(accessToken);
            ExecuteAsyncBatchRequests(accessToken);
        }

        private void ExecuteSyncBatchRequests(string accessToken)
        {
            var fb = new FacebookClient(accessToken);

            try
            {
                dynamic result = fb.Batch(
                    new FacebookBatchParameter("/me"),
                    new FacebookBatchParameter("/me/friends", new Dictionary<string, object> { { "limit", 10 } }),
                    new FacebookBatchParameter("/me/error"));

                dynamic result0 = result[0];
                dynamic result1 = result[1];
                dynamic result2 = result[2];

                // NOTE: always remeber to check for exception for single batch requests first.
                // you might actually want to process the result simultaneously. 
                if (result0 is Exception)
                {
                    // handle exception
                }
                else
                {
                    var name = result0.name;
                }

                if (result1 is Exception)
                {

                }
                else
                {

                }

                if (result2 is Exception)
                {

                }
                else
                {

                }
            }
            catch (FacebookApiException e)
            {
                // incase your access token is invalid or no net connection you might get general error.
                // so make sure to check for exception here also.
                throw;
            }
        }

        private void ExecuteAsyncBatchRequests(string accessToken)
        {
            // note: fb allows you to make maximum of 20 requests at a time.

            var fb = new FacebookClient(accessToken);

            // since batch reqeust is actually a normal POST request,
            // PostCompleted event will be fired.
            fb.PostCompleted += new EventHandler<FacebookApiEventArgs>(fb_PostCompleted);

            // as of v5.0.8 Facebook C# SDK supports all features of BatchRequests except for uploading binary files.
            fb.BatchAsync(new[]{
                new FacebookBatchParameter { HttpMethod = HttpMethod.Get, Path = "/4" },
                new FacebookBatchParameter(HttpMethod.Get, "/me/friend", new Dictionary<string, object> { { "limit", 10 } }),
                new FacebookBatchParameter("/me/friends", new { limit = 1 }) { Data = new { name = "one-friend", omit_response_on_success = false } },
                new FacebookBatchParameter { Parameters = new { ids = "{result=one-friend:$.data.0.id}" } },
                new FacebookBatchParameter("{result=one-friend:$.data.0.id}/feed", new { limit = 5 }),
                new FacebookBatchParameter("/me"),
                //new FacebookBatchParameter(HttpMethod.Post, "/me/feed", new { message = "test status update" }),
                new FacebookBatchParameter().Query("SELECT name FROM user WHERE uid="),
                new FacebookBatchParameter().Query(
                    "SELECT first_name FROM user WHERE uid=me()",
                    "SELECT last_name FROM user WHERE uid=me()"),
                new FacebookBatchParameter("/me/error")});
        }

        void fb_PostCompleted(object sender, FacebookApiEventArgs e)
        {
            // check general error first

            if (e.Error == null)
            {
                dynamic result = e.GetResultData();

                // don't call the generic version as it will throw error.
                // for batch requests only non generic version is supported.
                //dynamic resultT = e.GetResultData<T>();

                dynamic result0 = result[0];
                dynamic result1 = result[1];
                dynamic result2 = result[2];

                dynamic result5 = result[5];

                // NOTE: always remeber to check for exception for single batch requests first.
                // you might actually want to process the result simultaneously by creating threads. 
                if (result0 is Exception)
                {
                    // handle exception
                }
                else
                {
                    var name = result0.name;
                }

                if (result1 is Exception)
                {

                }
                else
                {

                }

                if (result2 is Exception)
                {

                }
                else
                {

                }

                if (result5 is Exception)
                {
                    // do something with the exception.
                }
                else
                {
                    MessageBox.Show("Hi " + result5.name);
                }
            }
            else
            {
                // do something with e.Error
            }
        }
    }
}
